      PROGRAM NAMEIO 
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          NAMEIO           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     PROGRAM :
C*          NAMEIO - NAME-ORIENTED INPUT/OUTPUT PACKAGE
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          PROVIDE A PORTABLE MEANS TO ASSIGN VALUES TO VARIABLES BY
C*          NAME (E.G., LIKE NAMELIST).   SEE DOCUMENT:NAMEIO.DOC
C*
C*     COMMON BLOCKS :
C*          NAMEIO.CMN
C*
C*     SUBPROGRAM REFERENCES :
C*          INIT, POPLINE, FINIS, GETSTM, DOCOM, DOINT, DOREAL, DOLOG, 
C*          DOCHAR, DODIM, DOPAR, DOINC, ERROR, PUTSTM
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          THE INPUT FILE MUST BE ARRANGED SO THAT ALL RELEVANT
C*          SPECIFICATION STATEMENTS COME AFTER THE COMMON STATEMENT
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*           5-NOV-1990  -  EFFICIENCY, EXIT HANDLER, FIXED CHARACTER 
C*                           BUGS
C*          15-AUG-1988  -  ADDED THE REST OF THE INTENDED SPECIFICATION
C*                            STATEMENTS
C*          21-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'NAMEIO.CMN'
      CHARACTER *5 LABEL
C
      EOF = .FALSE.
      CALL INIT
C
C --- IF AN END OF FILE WAS REACHED, SEE IF WE WERE IN AN INCLUDED FILE
C
10    IF (EOF) THEN
         IF (NREAD .GT. 60) THEN
            NREAD = NREAD - 1
            CALL POPLINE
            GO TO 10
         ELSE
C
C --- OTHERWISE, TIME TO FINISH UP
C
            CALL FINIS
         ENDIF
      ENDIF
C
      CALL GETSTM
      TYPE = STMT(1:1)
      IF ((TYPE .EQ. 'I') .AND. (STMT(3:3) .EQ. 'C')) TYPE = 'N'
      IF ((TYPE .EQ. 'C') .AND. (STMT(2:2) .EQ. 'H')) TYPE = 'S'
C
C --- COMMON BLOCK
C
      IF (TYPE .EQ. 'C') THEN
         CALL DOCOM
C
C --- 'INTEGER' TYPE STATEMENT
C
      ELSE IF (TYPE .EQ. 'I') THEN
         CALL DOINT
C
C --- 'REAL' TYPE STATEMENT
C
      ELSE IF (TYPE .EQ. 'R') THEN
         CALL DOREAL
C
C --- 'LOGICAL' TYPE STATEMENT
C
      ELSE IF (TYPE .EQ. 'L') THEN
         CALL DOLOG
C
C --- 'CHARACTER *' TYPE STATEMENT
C
      ELSE IF (TYPE .EQ. 'S') THEN
         CALL DOCHAR
C
C --- 'DIMENSION' STATEMENT
C
      ELSE IF (TYPE .EQ. 'D') THEN
         CALL DODIM
C
C --- 'PARAMETER' STATEMENT
C
      ELSE IF (TYPE .EQ. 'P') THEN
         CALL DOPAR
C
C --- 'INCLUDE' STATEMENT
C
      ELSE IF (TYPE .EQ. 'N') THEN
         CALL DOINC
C
      ELSE IF (TYPE .NE. ' ') THEN
         CALL ERROR ('Illegal statement ignored.')
         CALL PUTSTM (6, LABEL, STMT)
      ENDIF
      GO TO 10
      END
C
C---END NAMEIO
C
      SUBROUTINE INIT
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **           INIT            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          INIT
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          INITIALIZE PROGRAM-WIDE DATA, CHANGE DEFAULT SETTINGS
C*
C*     COMMON BLOCKS :
C*          NAMEIO.CMN, STACK, PARAMS
C*
C*     SUBPROGRAM REFERENCES :
C*          CAPS, BLANKS, RIGHT
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*           5-NOV-1990  -  ADDED 'HANDLER=' INITIALIZATION
C*          21-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'NAMEIO.CMN'
      COMMON / STACK / LINES(20), ENDS(20), NSTACK
      CHARACTER *(LC) LINES, LINE
      CHARACTER *10 WORK
      LOGICAL ENDS, ISALPHA
      COMMON / PARAMS / NAMES(1000), IVALUE(1000), NUMPAR
      CHARACTER *20 NAMES
C
C --- DEFAULT VALUES
C
      NCOM   = 0
      NCHAR  = 8
      SIGNAL = '&'
      NSTACK = 0
      NIN    = 5
      NOUT   = 6
      NREAD  = 60
      NSAVE  = 81
      MAX19  = 300
      EHAND  = 'EXIT'
C
      OPEN (UNIT=NREAD, READONLY, STATUS='OLD')
      READ (NREAD,910) LINE
      CLOSE (UNIT=NREAD)
      CALL CAPS (LINE)
      CALL BLANKS (LINE)
C
C --- GET NEW DEFAULT VALUES IF REQUESTED
C
      IF (LINE(1:2) .EQ. '!!') THEN
C
C ----- SIGNAL CHARACTER
C
         I = INDEX(LINE,'SIGNAL=')
         IF (I .NE. 0) SIGNAL = LINE(I+7:I+7)
C
C ----- INPUT UNIT NUMBER
C
         I = INDEX(LINE,'IN=')
         IF (I .NE. 0) THEN
            WORK = ' '
            I = I+3
            J = I
5           WORK(J-I+1:J-I+1) = LINE(J:J)
            J = J + 1
            IF ((LINE(J:J) .GE. '0') .AND. (LINE(J:J) .LE. '9')) GO TO 5
            CALL RIGHT(WORK)
            READ(WORK,900)NIN
         ENDIF
C
C ----- OUTPUT UNIT NUMBER
C
         I = INDEX(LINE,'OUT=')
         IF (I .NE. 0) THEN
            WORK = ' '
            I = I+4
            J = I
7           WORK(J-I+1:J-I+1) = LINE(J:J)
            J = J + 1
            IF ((LINE(J:J) .GE. '0') .AND. (LINE(J:J) .LE. '9')) GO TO 7
            CALL RIGHT(WORK)
            READ(WORK,900) NOUT
         ENDIF
C
C ----- NUMBER OF CHARACTERS PERMITTED IN A VARIABLE NAME
C
         I = INDEX(LINE,'NCHAR=')
         IF (I .NE. 0) THEN
            WORK = ' '
            I = I+6
            J = I
8           WORK(J-I+1:J-I+1) = LINE(J:J)
            J = J + 1
            IF ((LINE(J:J) .GE. '0') .AND. (LINE(J:J) .LE. '9')) GO TO 8
            CALL RIGHT(WORK)
            READ(WORK,900)NCHAR
         ENDIF
C
C ----- MAX 19
C
         I = INDEX(LINE,'MAX19=')
         IF (I .NE. 0) THEN
            WORK = ' '
            I = I+6
            J = I
9           WORK(J-I+1:J-I+1) = LINE(J:J)
            J = J + 1
            IF ((LINE(J:J) .GE. '0') .AND. (LINE(J:J) .LE. '9')) GO TO 9
            CALL RIGHT(WORK)
            READ(WORK,900) MAX19
         ENDIF
C
C ----- ERROR HANDLER
C
         I = INDEX(LINE,'HANDLER=')
         IF (I .NE. 0) THEN
            EHAND = ' '
            I = I+8
            J = 1
99          EHAND(J:J) = LINE(I:I)
            J = J + 1
            I = I + 1
            IF (ISALPHA(LINE(I:I)) .AND. (J .LE. 20)) GO TO 99
         ENDIF
C
         CALL GETSTM
      ENDIF
C
C --- INITIAL VALUES
C
      DO 20 I = 1,MAXCOM
         DO 10 J = 1,MAXVAR
            VTYPES(J,I) = ' '
            VNAMES(J,I) = ' '
10          CONTINUE
         ISCHAR(I) = .FALSE.
20       CONTINUE
      NUMPAR = 0
      RETURN
900   FORMAT(I10)
910   FORMAT(A)
      END
C
C---END INIT
C
      SUBROUTINE GETSTM 
C* 
C*                  ******************************* 
C*                  ******************************* 
C*                  **                           ** 
C*                  **          GETSTM           ** 
C*                  **                           ** 
C*                  ******************************* 
C*                  ******************************* 
C* 
C*     SUBPROGRAM : 
C*          GET STATEMENT 
C* 
C*     AUTHOR : 
C*          ART RAGOSTA 
C*          MS 219-1
C*          AMES RESEARCH CENTER 
C*          MOFFETT FIELD, CALIF 94035 
C*          (415) 604-5558 
C* 
C*     PURPOSE : 
C*          RETRIEVE THE NEXT, FULL FORTRAN STATEMENT.  GETSMT
C*          DOES NOT RETURN BLANK CARDS OR COMMENTS.
C*
C*     INPUT ARGUMENTS :
C*          NREAD - LOGICAL UNIT NUMBER ASSIGNED TO SOURCE CODE
C*
C*     OUTPUT ARGUMENTS :
C*          STMT  - THE STATEMENT (MINUS BLANKS, TABS)
C*          LENST - LENGTH OF STMT
C* 
C*     COMMON BLOCKS : 
C*          NAMEIO.CMN
C* 
C*     SUBPROGRAM REFERENCES : 
C*          MLIB_GETCRD, CAPITAL
C*
C*     RESTRICTIONS :
C*          WILL NOT HANDLE A STATEMENT WITH AN EXCLAMATION POINT IN A
C* 
C*     LANGUAGE AND COMPILER : 
C*          ANSI FORTRAN 77
C* 
C*     CHANGE HISTORY :
C*          08/15/88     MODIFIED FOR NAMEIO FOR SPECIAL EOF HANDLING
C*          07/11/88     ADDED DEC TAB SOURCE FORMAT
C*          04/26/88     CLEANED UP FOR MERLIB. FIXED ! HANDLING.
C*          09/15/86     MODIFIED TO NOT REMOVE BLANKS FROM CHARACTER
C*                         STRINGS
C*          01/24/84     CONVERTED TO VAX
C*          10/13/83     INITIAL VERSION 
C*                 
C***********************************************************************
C* 
      INCLUDE 'NAMEIO.CMN'
      LOGICAL NOT_INIT
      DATA NOT_INIT/.TRUE./
C
C --- FIRST TIME THROUGH WE WON'T HAVE A LOOK-AHEAD CARD, GET ONE
C
      IF ( NOT_INIT ) THEN
         EOF = .FALSE.
         CALL MLIB_GETCRD ( NREAD, CARD, EOF )
         NOT_INIT = .FALSE.
      ENDIF
C
      STMT   = ' '
      IPTR   = 1
C
C --- COPY ONLY NON-BLANK CHARACTERS, EXCEPT FOR CHARACTER STRINGS
C
10       I = 7
13       IF ((CARD(I:I) .NE. '''') .AND. (CARD(I:I) .NE. ' ') .AND.
     $       (CARD(I:I) .NE. CHAR(9))) THEN
            IF (CARD(I:I) .EQ. '!') GO TO 20    ! EXCLAMATION POINT COMMENT
            CALL CAPITAL(CARD(I:I))
            STMT(IPTR:IPTR) = CARD(I:I)
            IPTR = IPTR + 1
C
C --- QUOTED STRING COPIED VERBATIM
C
         ELSE IF (CARD(I:I) .EQ. '''') THEN
            STMT(IPTR:IPTR) = CARD(I:I)
            IPTR = IPTR + 1
C
C --- CHARACTER STRING, COPY, INCLUDING BLANKS, UNTIL MATCHING "'"
C
  15        I = I + 1
            IF (I .EQ. (LC+1)) THEN
               CALL MLIB_GETCRD ( NREAD, CARD, EOF )
               IF (CARD(6:6) .EQ. ' ')THEN
                  CALL MLIB_ERROR(2,'GETSTM',
     $               'Close quote missing on character string.')
                  GO TO 30
               ENDIF
               I = 7
            ENDIF
            STMT(IPTR:IPTR) = CARD(I:I)
            IPTR = IPTR + 1
            IF(CARD(I:I) .NE. '''') GO TO 15
         ENDIF
         I = I + 1
         IF (I .LE. LC) GO TO  13
C 
C --- CONTINUE COPYING IF THERE IS A CONTINUATION CARD
C
20    CALL MLIB_GETCRD ( NREAD, CARD, EOF )
      IF (CARD(6:6) .NE. ' ') GO TO 10
30    RETURN 
      END                  
C
C---END GETSTM
C
      SUBROUTINE MLIB_GETCRD ( NREAD, CARD, EOF )
C* 
C*                  ******************************* 
C*                  ******************************* 
C*                  **                           ** 
C*                  **        MLIB_GETCRD        ** 
C*                  **                           ** 
C*                  ******************************* 
C*                  ******************************* 
C* 
C*     SUBPROGRAM : 
C*          GET CARD 
C* 
C*     AUTHOR : 
C*          ART RAGOSTA 
C*          MS 207-5 
C*          AMES RESEARCH CENTER 
C*          MOFFETT FIELD, CALIF 94035 
C*          (415) 604-5558 
C* 
C*     PURPOSE : 
C*          RETRIEVE THE NEXT NON-COMMENT CARD. 
C* 
C*     INPUT ARGUMENTS :
C*          NREAD - THE LOGICAL UNIT NUMBER FOR THE SOURCE CODE
C*          EOF   - TRUE IF AN END OF FILE WAS ENCOUNTERED ON PREVIOUS
C*                   READ
C*
C*     OUTPUT ARGUMENTS :
C*          EOF  - TRUE IF AN END OF FILE OCCURRED
C*          CARD - THE CARD READ
C*
C*     COMMON BLOCKS : 
C*          NONE
C* 
C*     FILE REFERENCES : 
C*          NREAD
C* 
C*     SUBPROGRAM REFERENCES : 
C*          UNTAB, MLIB_ERROR
C* 
C*     LANGUAGE AND COMPILER : 
C*          ANSI FORTRAN 77
C* 
C*     VERSION AND DATE : 
C*          VERSION II.4
C* 
C*     CHANGE HISTORY : 
C*          04/26/88     GENERALIZED FOR MERLIB
C*          12/26/84     SAVE AND RESTORE OPTIONS ADDED
C*          01/24/84     VAX VERSION
C*          10/13/83     INITIAL VERSION 
C* 
C***********************************************************************
C* 
      PARAMETER (LC=132)
      CHARACTER *(LC) CARD
      LOGICAL EOF
C
      IF (EOF) CALL MLIB_ERROR 
     $          (3,'GETCRD','Attempt to read past end of file.')
10    READ ( NREAD, 900, END=100 ) CARD
C
C --- SKIP COMMENT AND BLANK CARDS
C
      IF ((CARD(1:1) .EQ. 'C') .OR. (CARD(1:1) .EQ. '*') .OR.
     $    (CARD(1:1) .EQ. 'c')) GO TO 10
C
C --- CHECK FOR VAX FORTRAN 'D_LINES' EXTENSION
C
      IF ((CARD(1:1) .EQ. 'D') .OR. (CARD(1:1) .EQ. 'd')) 
     $     CARD(1:1) = ' '
C
C --- CHECK FOR VAX TAB CONTINUATION EXTENSION
C
      IF (CARD(1:1) .EQ. CHAR(9)) THEN
         IF ((CARD(2:2) .GE. '1') .AND. (CARD(2:2) .LE. '9')) THEN
            CARD = '     $' // CARD(3:)
         ELSE
            CARD = '      ' // CARD(2:)
         ENDIF
      ENDIF
      IF (CARD .EQ. ' ') GO TO 10
      RETURN
C
100   EOF = .TRUE.
      CARD = ' '
      RETURN
900   FORMAT ( A )
      END
C
C---END MLIB_GETCRD
C
      SUBROUTINE DOCOM
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **           DOCOM           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          DOCOM
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          PROCESSING FOR COMMON STATEMENTS... EXTRACT ALL NAMES AND
C*          INITIALIZE TYPE, SIZE, ETC.
C*
C*     COMMON BLOCKS :
C*          NAMEIO.CMN
C*
C*     SUBPROGRAM REFERENCES :
C*          LENGTH, FATAL, GETVAR
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          NONE
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  22-JAN-1988
C*
C*     CHANGE HISTORY :
C*          22-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'NAMEIO.CMN'
      CHARACTER *20 CNAME, VNAME
C
      LS = LENGTH(STMT)
C
C --- GET COMMON NAME
C
      I = INDEX(STMT,'/')
      IF (I .EQ. 0) THEN
         CNAME = '$BLANK'
         IPTR = 7
      ELSE
         I = I + 1
         J = I
10       J = J + 1
         IF (STMT(J:J) .NE. '/') GO TO 10
         IF (J .EQ. I) THEN
            CNAME = '$BLANK'
         ELSE
            CNAME = STMT(I:J-1)
         ENDIF
         IPTR = J + 1
      ENDIF
C
C --- ADD NAME TO LIST
C
      IF (NCOM .GE. MAXCOM) CALL FATAL('Too many commons in input.')
      NCOM = NCOM + 1
      CNAMES(NCOM) = CNAME
C
C --- GET THE VARIABLE NAMES
C
      NVAR = 0
20    CALL GETVAR(IPTR,VNAME,ISIZE)
      NVAR = NVAR + 1
      VNAMES(NVAR,NCOM) = VNAME
      VSIZES(NVAR,NCOM) = ISIZE
C
C --- SET DEFAULT TYPE BY THE FIRST LETTER OF THE NAME
C
      IF ((VNAME(1:1) .GE. 'I') .AND. (VNAME(1:1) .LE. 'N')) THEN
         VTYPES(NVAR,NCOM) = 'I'
      ELSE
         VTYPES(NVAR,NCOM) = 'R'
      ENDIF
      IF (IPTR .LE. LS) GO TO 20
C
C ---  NUMBER OF VARIABLES THIS COMMON
C
      NUMVAR(NCOM) = NVAR
      RETURN
      END
C
C---END DOCOM
C
      SUBROUTINE DOINT
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **           DOINT           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          DOINT
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          PROCESS AN INTEGER STATEMENT... REPLACE TYPE OF EACH
C*           VARIABLE
C*
C*     COMMON BLOCKS :
C*          NAMECOM
C*
C*     SUBPROGRAM REFERENCES :
C*          LENGTH, GETVAR, ERROR
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          NONE
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  22-JAN-1988
C*
C*     CHANGE HISTORY :
C*          22-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'NAMEIO.CMN'
      CHARACTER *20 VNAME
C
      IPTR = 8
      LS = LENGTH (STMT)
10    CALL GETVAR (IPTR, VNAME, ISIZE)
C
C --- SEARCH FOR VARIABLE NAME (GO BACKWARDS THRU COMMONS FOR EFFICIENCY)
C
      DO 25 M = NCOM, 1, -1
         DO 20 I = 1,NUMVAR(M)
            IF (VNAMES(I,M) .EQ. VNAME) THEN
               VTYPES(I,M) = 'I'
               IF (ISIZE .NE. 1) VSIZES(I,M) = ISIZE
               GO TO 30
            ENDIF
20          CONTINUE
25       CONTINUE
C
C --- NAME NOT PREVIOUSLY DEFINED IN COMMON... ERROR
C
      CALL ERROR('Variable name ' // VNAME(1:LENGTH(VNAME)) //
     $           ' not found in COMMON.')
C
C --- MORE NAMES ON LINE ?
C
30    IF (IPTR .LE. LS) GO TO 10
      RETURN
      END
C
C---END DOINT
C
      SUBROUTINE DOREAL
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          DOREAL           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          DOREAL
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          PROCESS REAL STATEMENT... CHANGE TYPE OF VARIABLES
C*
C*     COMMON BLOCKS :
C*          NAMECOM
C*
C*     SUBPROGRAM REFERENCES :
C*          LENGTH, GETVAR, ERROR
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          NONE
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  22-JAN-1988
C*
C*     CHANGE HISTORY :
C*          22-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'NAMEIO.CMN'
      CHARACTER *20 VNAME
C
      IPTR = 5
      LS = LENGTH(STMT)
10    CALL GETVAR(IPTR,VNAME,ISIZE)
C
C --- SEARCH FOR VARIABLE NAME (GO BACKWARDS THRU COMMONS FOR EFFICIENCY)
C
      DO 25 M = NCOM, 1, -1
         DO 20 I = 1,NUMVAR(M)
            IF (VNAMES(I,M) .EQ. VNAME) THEN
               VTYPES(I,M) = 'R'
               IF (ISIZE .NE. 1) VSIZES(I,M) = ISIZE
               GO TO 30
            ENDIF
20          CONTINUE
25       CONTINUE
C
C --- VARIABLE MUST BE DEFINED IN COMMON
C
      CALL ERROR ('Variable name ' // VNAME(1:LENGTH(VNAME)) // 
     $            ' not found in COMMON.')
C
C --- MORE ENTRIES ON LINE?
C
30    IF (IPTR .LE. LS) GO TO 10
      RETURN
      END
C
C---END DOREAL
C
      SUBROUTINE DOLOG
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **           DOLOG           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          DOLOG
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          PROCESS LOGICAL STATEMENTS... REPLACE TYPE OF VARIABLES
C*
C*     COMMON BLOCKS :
C*          NAMECOM
C*
C*     SUBPROGRAM REFERENCES :
C*          LENGTH, GETVAR, ERROR
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          NONE
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  22-JAN-1988
C*
C*     CHANGE HISTORY :
C*          22-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'NAMEIO.CMN'
      CHARACTER *20 VNAME
C
      IPTR = 8
      LS = LENGTH(STMT)
10    CALL GETVAR(IPTR,VNAME,ISIZE)
C
C --- SEARCH FOR VARIABLE NAME (GO BACKWARDS THRU COMMONS FOR EFFICIENCY)
C
      DO 25 M = NCOM, 1, -1
         DO 20 I = 1,NUMVAR(M)
            IF (VNAMES(I,M) .EQ. VNAME) THEN
               VTYPES(I,M) = 'L'
               IF (ISIZE .NE. 1) VSIZES(I,M) = ISIZE
               GO TO 30
            ENDIF
20          CONTINUE
25       CONTINUE
C
C --- NAME MUST HAVE BEEN DECLARED IN COMMON
C
      CALL ERROR ('Variable name ' // VNAME(1:LENGTH(VNAME)) // 
     $            ' not found in COMMON.')
C
C --- CHECK FOR MORE NAMES ON LINE
C
30    IF (IPTR .LE. LS) GO TO 10
      RETURN
      END
C
C---END DOLOG                           
C                                                                       
      SUBROUTINE DOCHAR
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          DOCHAR           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          DOCHAR
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          PROCESS A CHARACTER STATEMENT... REPLACE TYPE AND SIZE
C*
C*     COMMON BLOCKS :
C*          NAMECOM
C*
C*     SUBPROGRAM REFERENCES :
C*          RIGHT, LENGTH, GETVAR, ERROR
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          NONE
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  22-JAN-1988
C*
C*     CHANGE HISTORY :
C*          22-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'NAMEIO.CMN'
      CHARACTER *20 VNAME
C
      IPTR = 10
C
C --- EXTRACT LENGTH SPECIFIER (DEFAULT TO 1 IF NOT PRESENT)
C
      IF (STMT(IPTR:IPTR) .EQ. '*') THEN
         IPTR = IPTR + 1
         CALL GETINT (STMT, IPTR, ISIZE)
      ELSE
         ISIZE = 1
      ENDIF
C
      LS = LENGTH(STMT)
10    CALL GETVAR (IPTR, VNAME, IDUM)  ! IDUM IS ARRAY SIZE
C
C --- SEARCH FOR VARIABLE NAME (GO BACKWARDS THRU COMMONS FOR EFFICIENCY)
C
      DO 25 M = NCOM, 1, -1
         DO 20 I = 1,NUMVAR(M)
            IF (VNAMES(I,M) .EQ. VNAME) THEN
               VTYPES(I,M) = 'C'
               IF (IDUM .NE. 1) THEN     ! LENGTH SPECIFIER ON VARIABLE?
                  VSIZES(I,M) = IDUM
               ELSE
                  VSIZES(I,M) = ISIZE    ! USE PRESENT DEFAULT SIZE
               ENDIF
               ISCHAR(M) = .TRUE.
               GO TO 30
            ENDIF
20          CONTINUE
25       CONTINUE
C
C --- NOT IN COMMON
C
      CALL ERROR ('Variable name ' // VNAME(1:LENGTH(VNAME)) //
     $            ' not found in COMMON.')
C
C --- CHECK FOR MORE NAMES ON LINE
C
30    IF (IPTR .LE. LS) GO TO 10
      RETURN
      END
C
C---END DOCHAR
C
      SUBROUTINE DODIM
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **           DODIM           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          DODIM
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          PROCESS A DIMENSION STATEMENT... CHANGE SIZE
C*
C*     COMMON BLOCKS :
C*          NAMECOM
C*
C*     SUBPROGRAM REFERENCES :
C*          LENGTH, GETVAR, ERROR
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          NONE
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  22-JAN-1988
C*
C*     CHANGE HISTORY :
C*          22-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'NAMEIO.CMN'
      CHARACTER *20 VNAME
C
      IPTR = 10
      LS = LENGTH(STMT)
10    CALL GETVAR(IPTR,VNAME,ISIZE)
C
C --- SEARCH FOR VARIABLE NAME
C
      DO 25 M = NCOM, 1, -1
         DO 20 I = 1,NUMVAR(M)
            IF (VNAMES(I,M) .EQ. VNAME) THEN
               VSIZES(I,M) = ISIZE
               GO TO 30
            ENDIF
20          CONTINUE
25       CONTINUE
C
C --- NAME NOT FOUND
C
      CALL ERROR ('Variable name ' // VNAME(1:LENGTH(VNAME)) //
     $            ' not found in COMMON.')
C
C --- CHECK FOR MORE ENTRIES ON LINE
C
30    IF (IPTR .LE. LS) GO TO 10
      RETURN
      END
C
C---END DODIM
C
      SUBROUTINE DOPAR
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **           DOPAR           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          PROCESS PARAMETER STATEMENT
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          GET PARAMETER VALUES FOR LATER USE
C*
C*     COMMON BLOCKS :
C*          NAMEIO.CMN, PARAMS
C*
C*     SUBPROGRAM REFERENCES :
C*          GETALP,  FATAL,  GETINT,  PUTSTM
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          NONE
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  14-JUL-1988
C*
C*     CHANGE HISTORY :
C*          14-JUL-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'NAMEIO.CMN'
      COMMON / PARAMS / NAMES(1000), IVALUE(1000), NUMPAR
      CHARACTER *20 NAMES, TEMP
C
C --- FORMAT IS:    PARAMETER (VAR=VALUE)
C
      I = INDEX (STMT,'(') + 1
      IF (I .EQ. 1) THEN
         CALL PUTSTM (NOUT, '     ', STMT)
         CALL FATAL ('Where''s the left paren? ')
      ENDIF
C
C --- GET PARAMETER NAME INTO "TEMP"
C
10    CALL GETALP (STMT, I, TEMP)
      IF (STMT(I:I) .NE. '=') THEN
         CALL PUTSTM (NOUT, '     ', STMT)
         CALL FATAL ('Where''s the "=" ? ')
      ENDIF
      I = I + 1
C
C --- INTEGER VALUES ONLY IN NAMEIO
C
      CALL GETINT (STMT, I, J)
      NUMPAR = NUMPAR + 1
      NAMES(NUMPAR) = TEMP
      IVALUE(NUMPAR) = J
C
C --- MORE VALUES ?
C
      IF (STMT(I:I) .EQ. ',') THEN
         I = I + 1
         GO TO 10
      ENDIF
      RETURN
      END
C
C---END DOPAR
C
      SUBROUTINE DOINC
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **           DOINC           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          DOINC
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          PROCESS AN 'INCLUDE' STATEMENT.  TRANSFER I/O CONTROL 
C*          TO NEW FILE.
C*
C*     COMMON BLOCKS :
C*          NAMECOM
C*
C*     SUBPROGRAM REFERENCES :
C*          LENGTH, ERROR
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          12-APR-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'NAMEIO.CMN'
      CHARACTER *80 FNAME
C
      LS = LENGTH(STMT)
      IS = 8
C
C --- REMOVE QUOTES FROM AROUND NAME OF INCLUDE FILE
C
      IF (STMT(IS:IS) .EQ. '''') IS = IS + 1
      IF (STMT(IS:IS) .EQ. '"')  IS = IS + 1
      IE = LS
      IF (STMT(IE:IE) .EQ. '''') IE = IE - 1
      IF (STMT(IE:IE) .EQ. '"')  IE = IE - 1
      IF (IE .LT. IS) THEN
         FNAME = '*' 
         GO TO 100
      ENDIF
C
C --- OPEN THE NEW FILE WITH A NEW LOGICAL UNIT NUMBER
C
      FNAME = STMT(IS:IE)
      NREAD = NREAD + 1
      OPEN (UNIT=NREAD, FILE=FNAME, READONLY, STATUS='OLD', ERR=100)
C
C --- THE LOOK-AHEAD CARD MUST BE SAVED FOR WHEN WE REACH EOF OF
C ---  THE INCLUDED FILE
C
      CALL PUSHLINE
      EOF  = .FALSE.
C
C --- GET NEW LOOK-AHEAD CARD
C
      CALL MLIB_GETCRD ( NREAD, CARD, EOF )
      RETURN
C
100   CALL ERROR ('File ' // FNAME(1:LENGTH(FNAME)) // ' not found.')
      NREAD = NREAD - 1
      RETURN
      END
C
C---END DOINC
C
      SUBROUTINE GETVAR ( IPTR, VNAME, ISIZE )
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          GETVAR           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GETVAR
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          EXTRACT A VARIABLE NAME FROM A STATEMENT
C*
C*     INPUT ARGUMENTS :
C*          IPTR  - LOCATION IN THE STATEMENT TO START THE SCAN
C*
C*     OUTPUT ARGUMENTS :
C*          VNAME - THE NAME OF THE VARIABLE
C*          ISIZE - THE DIMENSION OF THE VARIABLE
C*
C*     COMMON BLOCKS :
C*          NAMECOM
C*
C*     SUBPROGRAM REFERENCES :
C*          LENGTH, GETSUB
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          NONE
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  22-JAN-1988
C*
C*     CHANGE HISTORY :
C*          22-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'NAMEIO.CMN'
      CHARACTER *(*) VNAME
      DIMENSION IND(6)
C
      ISIZE = 1
      IV    = 1
      VNAME = ' '
C
C --- GET VARIABLE NAME
C
      CALL GETALP (STMT, IPTR, VNAME)
C
C --- NOT IN VARIABLE NAME, EITHER '(', '*',  OR ','
C
      IF ( STMT(IPTR:IPTR) .EQ. '(' ) THEN
C
C ------ SUBSCRIPT
C
         IPTR = IPTR + 1
         J    = 1
C
C  ------- GET NEXT SUBSCRIPT AND STORE IT
C
20       CALL GETSUB ( I, IPTR )
         IND(J) = I
         IF ( STMT(IPTR:IPTR) .EQ. ',' ) THEN
            J = J + 1
            IPTR = IPTR + 1
            GO TO 20
         ENDIF
C
C  ------  SKIP RIGHT PAREN
C
         IPTR = IPTR + 1
C
C  ------- GOT ALL SUBSCRIPTS, MULTIPLY THEM FOR TOTAL SIZE OF VARIABLE
C
         DO 30 I = 1,J
            ISIZE = ISIZE * IND(I)
30          CONTINUE
C
C --- ALTERNATE LENGTH SPECIFIER FOR CHARACTER DATA
C
      ELSE IF ( STMT(IPTR:IPTR) .EQ. '*' ) THEN
         IPTR = IPTR + 1
         CALL GETINT (STMT, IPTR, ISIZE)
      ENDIF
C
C --- BYPASS COMMA
C
      IPTR = IPTR + 1
      RETURN
      END
C
C---END GETVAR
C
      SUBROUTINE GETSUB (I, IPTR)
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          GETSUB           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GETSUB
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          EXTRACT A SUBSCRIPT FOR A VARIABLE
C*
C*     INPUT ARGUMENTS :
C*          IPTR  - LOCATION IN STATEMENT TO START SCAN
C*
C*     OUTPUT ARGUMENTS :
C*          I  - THE VALUE OF THE SUBSCRIPT
C*
C*     COMMON BLOCKS :
C*          NAMECOM
C*
C*     SUBPROGRAM REFERENCES :
C*          ERROR
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          NONE
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  22-JAN-1988
C*
C*     CHANGE HISTORY :
C*          22-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'NAMEIO.CMN'
      COMMON / PARAMS / NAMES(1000), IVALUE(1000), NUMPAR
      CHARACTER *20 NAMES, TEMP
C
C --- INTEGER SUBSCRIPT
C
      IF ((STMT(IPTR:IPTR) .GE. '0') .AND. 
     $    (STMT(IPTR:IPTR) .LE. '9')) THEN
         I = 0
10       I = 10 * I + ICHAR(STMT(IPTR:IPTR)) - ICHAR('0')
         IPTR = IPTR + 1
         IF (( STMT(IPTR:IPTR) .GE. '0') .AND.
     $       ( STMT(IPTR:IPTR) .LE. '9')) GO TO 10
C
C --- NOT AN INTEGER SUBSCRIPT, THE ONLY OTHER VALID ENTITY HERE IS
C ---  A PARAMETER NAME... CHECK IT OUT
C
      ELSE
C
C ----- GET NAME OF PARAMETER
C
         TEMP = ' '
         II = 0
20       IF (((STMT(IPTR:IPTR) .GE. '0')  .AND. 
     $        (STMT(IPTR:IPTR) .LE. '9')) .OR.
     $       ((STMT(IPTR:IPTR) .GE. 'A')  .AND.
     $        (STMT(IPTR:IPTR) .LE. 'Z')) .OR.
     $        (STMT(IPTR:IPTR) .EQ. '$')  .OR.
     $        (STMT(IPTR:IPTR) .EQ. '_')) THEN
            II = II + 1
            IF (II .LE. 20) TEMP(II:II) = STMT(IPTR:IPTR)
            IPTR = IPTR + 1
            GO TO 20
         ENDIF
C
C ----- NOW SEE IF WE HAVE THIS PARAMETER DEFINED
C
         DO 30 II = 1, NUMPAR
            IF (TEMP .EQ. NAMES(II)) THEN
               I = IVALUE(II)
               RETURN
            ENDIF
30       CONTINUE
         CALL ERROR ('Undefined parameter '//TEMP)
         I = 1
      ENDIF
      RETURN
      END
C
C---END GETSUB
C
      SUBROUTINE FINIS
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **           FINIS           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          FINIS
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          FINISH UP PROCESSING... OUTPUT CODE
C*
C*     COMMON BLOCKS :
C*          NAMECOM, BIG
C*
C*     SUBPROGRAM REFERENCES :
C*          GETBIG, COMOUT, LENGTH
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          "CARRIAGECONTROL='LIST'" IS VAX SPECIFIC
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  22-JAN-1988
C*
C*     CHANGE HISTORY :
C*          22-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER (MAXALL=1000)
      INCLUDE 'NAMEIO.CMN'
      COMMON / BIG / NUMALL, WNAMES(MAXALL), WTYPES(MAXALL), 
     $  WSIZES(MAXALL), WINDEX(MAXALL), WCOM(MAXALL)
      CHARACTER *20 WNAMES
      CHARACTER *1 WTYPES
      INTEGER WSIZES, WINDEX, WCOM
      CHARACTER *80 ALINE
      CHARACTER *4 WORK
C
C
C --- INTEGRATE ALL DATA INTO SINGLE, LARGE ARRAYS
C
      CALL GETBIG
C
C --- THE FOLLOWING LINE IS VAX-SPECIFIC AND SHOULD BE REMOVED FOR
C ---  OTHER SYSTEMS.
C
      OPEN(UNIT=NSAVE,CARRIAGECONTROL='LIST',STATUS='NEW')
      WRITE (NSAVE,900) 
      CALL COMOUT
C
C --- CREATE BLOCK DATA
C
      WRITE(NSAVE,902) NIN, NOUT, SIGNAL, NUMALL
C
C --- NUMBER OF VARIABLES THAT WILL FIT ON 19 CONTINUATION LINES (PESSIMISTIC)
C
      NUM19  = MIN0(1200 / (NCHAR+3),MAX19)
      ISTART = 1
C
C --- VARIABLE NAMES
C
10    IEND   = MIN0(NUMALL, ISTART+NUM19-1)
      WRITE(ALINE,970) ISTART,IEND
      IL = LENGTH(ALINE)+1
      DO 30 I = ISTART, IEND
         ALINE(IL:IL+NCHAR+1) = ''''//WNAMES(I)(1:NCHAR)//''''
         IL = IL + NCHAR + 2
         IF (I .EQ. IEND) THEN
            ALINE(IL:IL) = '/'
            WRITE(NSAVE,910) ALINE(1:IL)
         ELSE
            ALINE(IL:IL) = ','
            IL = IL + 1
            IF (IL .GT. (72-(NCHAR+2))) THEN
               WRITE(NSAVE,910) ALINE(1:IL-1)
               ALINE = '     $ '
               IL = 8
            ENDIF
         ENDIF
30       CONTINUE
      IF (IEND .NE. NUMALL) THEN
         ISTART = IEND+1
         GO TO 10
      ENDIF
C
C --- VARIABLE SIZES
C
      ISTART = 1
      NUM19 = MIN0(230,MAX19)
35    IEND   = MIN0(NUMALL, ISTART+NUM19-1)
      WRITE(ALINE,980) ISTART,IEND
      IL = LENGTH(ALINE)+1
      DO 40 I = ISTART, IEND
         WRITE(WORK,920) WSIZES(I)
         ALINE(IL:IL+3) = WORK
         IL = IL + 4
         IF (I .EQ. IEND) THEN
            ALINE(IL:IL) = '/'
            WRITE(NSAVE,910) ALINE(1:IL)
         ELSE
            ALINE(IL:IL) = ','
            IL = IL + 1
            IF (IL .GT. 66) THEN
               WRITE(NSAVE,910) ALINE(1:IL-1)
               ALINE = '     $ '
               IL = 8
            ENDIF
         ENDIF
40       CONTINUE
      IF (IEND .NE. NUMALL) THEN
         ISTART = IEND+1
         GO TO 35
      ENDIF
C
C --- VARIABLE TYPES
C
      ISTART = 1
      NUM19 = MIN0(290,MAX19)
45    IEND   = MIN0(NUMALL, ISTART+NUM19-1)
      WRITE(ALINE,990) ISTART,IEND
      IL = LENGTH(ALINE)+1
      DO 50 I = ISTART, IEND
         ALINE(IL:IL+9) = ''''//WTYPES(I)//''''
         IL = IL + 3
         IF (I .EQ. IEND) THEN
            ALINE(IL:IL) = '/'
            WRITE(NSAVE,910) ALINE(1:IL)
         ELSE
            ALINE(IL:IL) = ','
            IL = IL + 1
            IF (IL .GT. 67) THEN
               WRITE(NSAVE,910) ALINE(1:IL-1)
               ALINE = '     $ '
               IL = 8
            ENDIF
         ENDIF
50       CONTINUE
      IF (IEND .NE. NUMALL) THEN
         ISTART = IEND+1
         GO TO 45
      ENDIF
C
C --- INDEX INTO COMMON BLOCK
C
      ISTART = 1
      NUM19 = MIN0(230,MAX19)
55    IEND   = MIN0(NUMALL, ISTART+NUM19-1)
      WRITE(ALINE,993) ISTART,IEND
      IL = LENGTH(ALINE)+1
      DO 60 I = ISTART, IEND
         WRITE(WORK,920) WINDEX(I)
         ALINE(IL:IL+3) = WORK
         IL = IL + 4
         IF (I .EQ. IEND) THEN
            ALINE(IL:IL) = '/'
            WRITE(NSAVE,910) ALINE(1:IL)
         ELSE
            ALINE(IL:IL) = ','
            IL = IL + 1
            IF (IL .GT. 66) THEN
               WRITE(NSAVE,910) ALINE(1:IL-1)
               ALINE = '     $ '
               IL = 8
            ENDIF
         ENDIF
60       CONTINUE
      IF (IEND .NE. NUMALL) THEN
         ISTART = IEND+1
         GO TO 55
      ENDIF
C
C --- COMMON BLOCK NUMBER
C
      ISTART = 1
      NUM19 = MIN0(230,MAX19)
62    IEND   = MIN0(NUMALL, ISTART+NUM19-1)
      WRITE(ALINE,996) ISTART,IEND
      IL = LENGTH(ALINE)+1
      DO 65 I = ISTART, IEND
         WRITE(WORK,920) WCOM(I)
         ALINE(IL:IL+3) = WORK
         IL = IL + 4
         IF (I .EQ. IEND) THEN
            ALINE(IL:IL) = '/'
            WRITE(NSAVE,910) ALINE(1:IL)
         ELSE
            ALINE(IL:IL) = ','
            IL = IL + 1
            IF (IL .GT. 66) THEN
               WRITE(NSAVE,910) ALINE(1:IL-1)
               ALINE = '     $ '
               IL = 8
            ENDIF
         ENDIF
65       CONTINUE
      IF (IEND .NE. NUMALL) THEN
         ISTART = IEND+1
         GO TO 62
      ENDIF
C
      WRITE(NSAVE,9000)
C
C --- MOST OF THE "READC", "WRITEV", AND RELATED ROUTINES ARE IN THE
C --- FILE "NAMESTUF.FOR".   COMMENTS IN THIS FILE STARTING WITH AN
C --- ASTERISK SIGNAL NAMEIO TO INSERT APPLICATION-SPECIFIC CODE.
C
      OPEN(UNIT=99,NAME='SYS$SYSTEM:NAMESTUF.FOR',STATUS='OLD',
     $   READONLY,ERR=1000)
70    READ(99,910,END=200) ALINE
C
C --- COMMON BLOCK (NAMEIO-INTERNAL)
C
      IF (ALINE(1:2) .EQ. '**') THEN
         CALL COMOUT
C
C --- COMMON BLOCK (APPLICATION-SPECIFIC)
C
      ELSE IF (ALINE(1:2) .EQ. '*1') THEN
         DO 80 I=1,NCOM
            IF (ISCHAR(I)) THEN
               WRITE(NSAVE,935) CNAMES(I)(1:LENGTH(CNAMES(I))),I,
     $          (VINDEX(NUMVAR(I),I)+VSIZES(NUMVAR(I),I)-1),I
            ELSE
               WRITE(NSAVE,930) CNAMES(I)(1:LENGTH(CNAMES(I))),I,
     $          (VINDEX(NUMVAR(I),I)+VSIZES(NUMVAR(I),I)-1)
            ENDIF
80          CONTINUE
C
C --- COMPUTED GOTO FOR "ASIGN0"
C
      ELSE IF (ALINE(1:2) .EQ. '*2') THEN
         STMT='GOTO('
         IS = 6
         DO 90 I=1,NCOM
            WRITE(WORK,920)I
            STMT(IS:IS+2) = WORK(3:4) // ','
            IS = IS + 3
90          CONTINUE
         STMT(IS-1:IS+7) = '),VCOM(J)'
         CALL PUTSTM (NSAVE, '     ', STMT)
         DO 95 I = 1,NCOM
            IF (ISCHAR(I)) THEN
               WRITE(NSAVE,941)I
            ELSE
               WRITE(NSAVE,940)I,I
            ENDIF
95          CONTINUE
C
C --- COMPUTED GOTO FOR "WRITEV"
C
      ELSE IF (ALINE(1:2) .EQ. '*3') THEN
         STMT='   GOTO('
         IS = 9
         DO 100 I=1,NCOM
            WRITE(WORK,920)I
            STMT(IS:IS+2) = WORK(3:4) // ','
            IS = IS + 3
100         CONTINUE
         STMT(IS-1:IS+7) = '),VCOM(M)'
         CALL PUTSTM (NSAVE, '        ', STMT)
         DO 105 I = 1,NCOM
            IF (ISCHAR(I)) THEN
               WRITE(NSAVE,955)I,I
            ELSE
               WRITE(NSAVE,950)I,I
            ENDIF
105         CONTINUE
C
C --- COMPUTED GOTO FOR "ASIGNC0"
C
      ELSE IF (ALINE(1:2) .EQ. '*5') THEN
         STMT='GOTO('
         IS = 6
         DO 110 I=1,NCOM
            WRITE(WORK,920)I
            STMT(IS:IS+2) = WORK(3:4) // ','
            IS = IS + 3
110         CONTINUE
         STMT(IS-1:IS+7) = '),VCOM(J)'
         CALL PUTSTM (NSAVE, '     ', STMT)
         DO 115 I = 1,NCOM
            IF (ISCHAR(I)) THEN
               WRITE(NSAVE,942)I,I
            ELSE
               WRITE(NSAVE,941)I
            ENDIF
115         CONTINUE
C
C --- WRITEC STUFF
C
      ELSE IF (ALINE(1:2) .EQ. '*4') THEN
         WRITE(NSAVE,9010) NCHAR, NCOM, NCOM
         ALINE = '      DATA CNAMES/'
         IL = 19
         DO 120 I = 1,NCOM
            ALINE(IL:IL+NCHAR+1) = ''''//CNAMES(I)(1:NCHAR)//''''
            IL = IL + NCHAR + 2
            IF (I .EQ. NCOM) THEN
               ALINE(IL:IL) = '/'
               WRITE(NSAVE,910) ALINE(1:LENGTH(ALINE))
            ELSE
               ALINE(IL:IL) = ','
               IL = IL + 1
               IF (IL .GT. (70-NCHAR)) THEN     ! 73-nchar-3
                  WRITE(NSAVE,910) ALINE(1:LENGTH(ALINE))
                  ALINE = '     $ '
                  IL = 8
               ENDIF
            ENDIF
120      CONTINUE
C
C --- EXIT HANDLER
C
      ELSE IF (ALINE(1:2) .EQ. '*6') THEN
         WRITE (NSAVE,9020) EHAND(1:LENGTH(EHAND))
C
C --- OTHERWISE JUST COPY THE LINE
C
      ELSE IF (ALINE(1:2) .EQ. '*7') THEN
         WRITE (NSAVE,9030) NIN
C
C --- OTHERWISE JUST COPY THE LINE
C
      ELSE
         WRITE(NSAVE,910) ALINE(1:LENGTH(ALINE))
      ENDIF
      GO TO 70
C
C --- END OF FILE
C
200   CALL EXIT
1000  CALL ERROR('Unable to open file SYS$SYSTEM:NAMESTUF.FOR.')
      STOP
900   FORMAT('      BLOCK DATA')
902   FORMAT('      DATA NIN/',I2,'/, NOUT/',I2,'/, SIGNAL/''',A1,'''/',
     $ ', NUMALL/',I5,'/'/
     $ '      DATA NAMERR/.FALSE./, DOINGC/.FALSE./, ECHO/.FALSE./')
905   FORMAT('      DATA NCOM/',I3,'/')
910   FORMAT(A)
920   FORMAT(I4)
930   FORMAT('      COMMON/',A,'/X',I2.2,'(',I4,')')
935   FORMAT('      COMMON/',A,'/X',I2.2,/,
     $       '      CHARACTER *',I5,'  X',I2.2)
940   FORMAT(I4,'  X',I2.2,'(VINDEX(J)+ISUB-1) = RV',/,
     $ '      GO TO 1000')
941   FORMAT(I4,'  GO TO 1000')
942   FORMAT(I4,'  X',I2.2,'(VINDEX(J):VINDEX(J)+VSIZES(J)-1) = SV',/,
     $ '      GO TO 1000')
950   FORMAT(I4,'     RV=X',I2.2,'(VINDEX(M)+K-1)',/,
     $ '         GO TO 400')
955   FORMAT(I4,'     CV=X',I2.2,'(VINDEX(M):VINDEX(M)+VSIZES(M)-1)',/,
     $ '         GO TO 400')
960   FORMAT(I4,'         X',I2.2,'=VALUE',/,
     $ '         GO TO 90')
970   FORMAT('      DATA (VNAMES(I),I=',I4,',',I4,')/')
980   FORMAT('      DATA (VSIZES(I),I=',I4,',',I4,')/')
990   FORMAT('      DATA (VTYPES(I),I=',I4,',',I4,')/')
993   FORMAT('      DATA (VINDEX(I),I=',I4,',',I4,')/')
996   FORMAT('      DATA (VCOM(I),I=',I4,',',I4,')/')
9000  FORMAT('      END',/,
     $       'C',/,
     $       'C---END BLOCK DATA',/,
     $       'C')
9010  FORMAT('      CHARACTER *(',I2,') CNAMES(',I2,')',/,
     $       '      DATA NUMCOM/',I2,'/')
9020  FORMAT('            CALL ',A)
9030  FORMAT('1000  IF (NIN .EQ. ',I2,') THEN')
      END
C
C---END FINIS
C
      SUBROUTINE FIXALL
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          FIXALL           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          FIXALL
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          AFTER ALL OF THE VARIABLES HAVE BEEN FOUND, IT IS NECESSARY
C*          TO GO THROUGH EACH COMMON BLOCK AND FIX THE INDICES SINCE
C*          THE SIZE OF A VARIABLE MAY HAVE BEEN CHANGED IN A DIMENSION
C*          OR TYPE SPECIFICATION STATEMENT.
C*
C*     INPUT ARGUMENTS :
C*          NONE
C*
C*     OUTPUT ARGUMENTS :
C*          NONE
C*
C*     COMMON BLOCKS :
C*          NAMEIO.CMN
C*
C*     SUBPROGRAM REFERENCES :
C*          NONE
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          NONE
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  31-AUG-1988
C*
C*     CHANGE HISTORY :
C*          31-AUG-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'NAMEIO.CMN'
C
C --- FIX UP INDEX ARRAY FOR EACH COMMON BLOCK SINCE THE SIZES
C ---  MAY HAVE BEEN CHANGED BY SPECIFICATION STATEMENTS
C
      DO 20 J = 1, NCOM
         IADR = 1
C
C ----- THE FIRST LOCATION IS ALWAYS 1...
C -----  EACH ADDITIONAL VARIABLE IS OFFSET BY THE SIZE OF THE 
C -----  PREVIOUS VARIABLE
C
         DO 30 I = 1, NUMVAR(J)
            VINDEX(I,J) = IADR
            IADR = IADR + VSIZES(I,J)
30          CONTINUE
20       CONTINUE
      RETURN
      END
C
C---END FIXALL
C
      SUBROUTINE COMOUT
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          COMOUT           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          COMOUT
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          OUTPUT THE NAMEIO-INTERNAL COMMON BLOCKS 
C*
C*     COMMON BLOCKS :
C*          NAMEIO.CMN, BIG
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          22-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER (MAXALL=1000)
      INCLUDE 'NAMEIO.CMN'
      COMMON / BIG / NUMALL, WNAMES(MAXALL), WTYPES(MAXALL), 
     $  WSIZES(MAXALL), WINDEX(MAXALL), WCOM(MAXALL)
      CHARACTER *20 WNAMES
      CHARACTER *1 WTYPES
      INTEGER WSIZES, WINDEX, WCOM
C
      WRITE(NSAVE,900) NUMALL, NUMALL, NUMALL, NUMALL, NUMALL, NCHAR
      RETURN
900   FORMAT(
     $'      COMMON / NMCOM0 / NIN, NOUT, NAMERR, ECHO'/
     $'      COMMON / NMCOM1 / NUMALL, ICARD, DOINGC, IBUFF,'/
     $'     $    VSIZES(',I4,'),   VCOM(',I4,'), VINDEX(',I4,')'/
     $'      COMMON / NMCOM2 / SIGNAL, BUFFER, VTYPES(',I4,'), ',
     $       'VNAMES(',I4,')'/
     $'      CHARACTER *',I2,' VNAMES'/
     $'      CHARACTER *132 BUFFER'/
     $'      CHARACTER *1 VTYPES, SIGNAL'/
     $'      INTEGER VSIZES, VINDEX, VCOM'/
     $'      LOGICAL NAMERR, DOINGC, ECHO')
      END
C
C---END COMOUT
C
      SUBROUTINE GETBIG
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          GETBIG           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GETBIG
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          COMBINE ALL DATA FROM INDIVIDUAL ARRAYS INTO ONE BIG ARRAY.
C*
C*     COMMON BLOCKS :
C*          NAMECOM, BIG
C*
C*     SUBPROGRAM REFERENCES :
C*          FIXALL, ISORT
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          NONE
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  22-JAN-1988
C*
C*     CHANGE HISTORY :
C*          22-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER (MAXALL=1000)
      INCLUDE 'NAMEIO.CMN'
      COMMON / BIG / NUMALL, WNAMES(MAXALL), WTYPES(MAXALL), 
     $  WSIZES(MAXALL), WINDEX(MAXALL), WCOM(MAXALL)
      CHARACTER *20 WNAMES
      CHARACTER *1 WTYPES, XTYPES(MAXALL)
      INTEGER WSIZES, WINDEX, WCOM, INDX(MAXALL)
      INTEGER XSIZES(MAXALL), XCOM(MAXALL), XINDEX(MAXALL)
C
      NUMALL = 0
      IW = 0
      CALL FIXALL
      DO 10 I=1,NCOM
         NUMALL = NUMALL + NUMVAR(I)
C
C --- CONVERT DATA FROM DOUBLY DIMENSIONED TO SINGLY
C
         DO 5 J = 1, NUMVAR(I)
            IW = IW + 1
            WNAMES(IW) = VNAMES(J,I)
            XTYPES(IW) = VTYPES(J,I)
            XSIZES(IW) = VSIZES(J,I)
            XINDEX(IW) = VINDEX(J,I)
            XCOM(IW) = I
5           CONTINUE
10       CONTINUE
C
C --- ALPHABETIZE
C
      CALL ISORT(WNAMES,NUMALL,INDX)
C
C
C --- FINISH SORT FOR ALL RELATED ARRAYS
C
      IW = 0
      DO 20 I = 1,NUMALL
         IW = IW + 1
         WTYPES(IW) = XTYPES(INDX(IW))
         WSIZES(IW) = XSIZES(INDX(IW))
         WINDEX(IW) = XINDEX(INDX(IW))
         WCOM(IW)   = XCOM(INDX(IW))
20       CONTINUE
      RETURN
      END
C
C---END GETBIG
C
      SUBROUTINE ERROR ( TEXT )
      INCLUDE 'NAMEIO.CMN'
      CHARACTER *(*) TEXT
C
      WRITE(NOUT,900) TEXT
      RETURN
900   FORMAT(' *** Error, ',A)
      END
C
C---END ERROR
C
      SUBROUTINE FATAL ( TEXT )
      INCLUDE 'NAMEIO.CMN'
      CHARACTER *(*) TEXT
C
      WRITE(NOUT,900) TEXT
      CALL EXIT
900   FORMAT(' ***',/,' *** Fatal error, ',A,/,' ***')
      END
C
C---END FATAL
C
      SUBROUTINE PUSHLINE 
C*
C*   USED TO STORE THE LOOK-AHEAD CARDS DURING INCLUDE PROCESSING
C*
      INCLUDE 'NAMEIO.CMN'
      COMMON / STACK / LINES(20), ENDS(20), NSTACK
      CHARACTER *(LC) LINES
      LOGICAL ENDS
C
      NSTACK = NSTACK + 1
      IF (NSTACK .GT. 20) CALL FATAL ('Stack overflow.')
      LINES(NSTACK) = CARD
      ENDS(NSTACK) = EOF
      RETURN
      END
C
C---END PUSHLINE
C
      SUBROUTINE POPLINE 
C*
C*   USED TO RESTORE THE LOOK-AHEAD CARDS AFTER INCLUDE PROCESSING
C*
      INCLUDE 'NAMEIO.CMN'
      COMMON / STACK / LINES(20), ENDS(20), NSTACK
      CHARACTER *(LC) LINES
      LOGICAL ENDS
C
      IF (NSTACK .LT. 1) CALL FATAL ( 'Stack undeflow.' )
      CARD = LINES(NSTACK)
      EOF  = ENDS(NSTACK)
      NSTACK = NSTACK - 1
      RETURN
      END
C
C---END POPLINE
C
      SUBROUTINE GETALP ( S, IPTR, TEMP )
C*
C*   GET THE NEXT ALPHA-NUMERIC TOKEN FROM THE STRING
C*
      CHARACTER *(*) S, TEMP
C
      TEMP = ' '
      II = 0
20    IF (((S(IPTR:IPTR) .GE. '0')  .AND. 
     $     (S(IPTR:IPTR) .LE. '9')) .OR.
     $    ((S(IPTR:IPTR) .GE. 'A')  .AND.
     $     (S(IPTR:IPTR) .LE. 'Z')) .OR.
     $     (S(IPTR:IPTR) .EQ. '$')  .OR.
     $     (S(IPTR:IPTR) .EQ. '_')) THEN
         II = II + 1
         IF (II .LE. LEN(TEMP)) TEMP(II:II) = S(IPTR:IPTR)
         IPTR = IPTR + 1
         IF (IPTR .LE. LEN(S)) GO TO 20
      ENDIF
      RETURN
      END
C
C---END GETALP
C
      SUBROUTINE GETINT ( S, I, J )
C*
C*   GET THE NEXT INTEGER TOKEN FROM THE STRING
C*
      CHARACTER *(*) S
C
      J = 0
10    IF ((S(I:I) .GE. '0') .AND. 
     $    (S(I:I) .LE. '9')) THEN
         J = 10 * J + ICHAR(S(I:I)) - ICHAR('0')
         I = I + 1
         GO TO 10
      ENDIF
      RETURN
      END
C
C--END GETINT
C
